@@ -31,9 +31,9 @@ var _ = Describe("Client", func() {
 		tracer          *mocklogging.MockConnectionTracer
 		config          *Config
 
-		originalClientSessConstructor func(
+		originalClientConnConstructor func(
 			conn sendConn,
-			runner sessionRunner,
+			runner connRunner,
 			destConnID protocol.ConnectionID,
 			srcConnID protocol.ConnectionID,
 			conf *Config,
@@ -45,19 +45,18 @@ var _ = Describe("Client", func() {
 			tracingID uint64,
 			logger utils.Logger,
 			v protocol.VersionNumber,
-		) quicSession
+		) quicConn
 	)
 
 	BeforeEach(func() {
 		tlsConf = &tls.Config{NextProtos: []string{"proto1"}}
 		connID = protocol.ConnectionID{0, 0, 0, 0, 0, 0, 0x13, 0x37}
-		originalClientSessConstructor = newClientSession
+		originalClientConnConstructor = newClientConnection
 		tracer = mocklogging.NewMockConnectionTracer(mockCtrl)
 		tr := mocklogging.NewMockTracer(mockCtrl)
 		tr.EXPECT().TracerForConnection(gomock.Any(), protocol.PerspectiveClient, gomock.Any()).Return(tracer).MaxTimes(1)
 		config = &Config{Tracer: tr, Versions: []protocol.VersionNumber{protocol.VersionTLS}}
-		Eventually(areSessionsRunning).Should(BeFalse())
-		// sess = NewMockQuicSession(mockCtrl)
+		Eventually(areConnsRunning).Should(BeFalse())
 		addr = &net.UDPAddr{IP: net.IPv4(192, 168, 100, 200), Port: 1337}
 		packetConn = NewMockPacketConn(mockCtrl)
 		packetConn.EXPECT().LocalAddr().Return(&net.UDPAddr{}).AnyTimes()
@@ -65,7 +64,7 @@ var _ = Describe("Client", func() {
 			srcConnID:  connID,
 			destConnID: connID,
 			version:    protocol.VersionTLS,
-			conn:       newSendPconn(packetConn, addr),
+			sconn:      newSendPconn(packetConn, addr),
 			tracer:     tracer,
 			logger:     utils.DefaultLogger,
 		}
@@ -78,33 +77,27 @@ var _ = Describe("Client", func() {
 
 	AfterEach(func() {
 		connMuxer = origMultiplexer
-		newClientSession = originalClientSessConstructor
+		newClientConnection = originalClientConnConstructor
 	})
 
 	AfterEach(func() {
-		if s, ok := cl.session.(*session); ok {
+		if s, ok := cl.conn.(*connection); ok {
 			s.shutdown()
 		}
-		Eventually(areSessionsRunning).Should(BeFalse())
+		Eventually(areConnsRunning).Should(BeFalse())
 	})
 
 	Context("Dialing", func() {
-		var origGenerateConnectionID func(int) (protocol.ConnectionID, error)
 		var origGenerateConnectionIDForInitial func() (protocol.ConnectionID, error)
 
 		BeforeEach(func() {
-			origGenerateConnectionID = generateConnectionID
 			origGenerateConnectionIDForInitial = generateConnectionIDForInitial
-			generateConnectionID = func(int) (protocol.ConnectionID, error) {
-				return connID, nil
-			}
 			generateConnectionIDForInitial = func() (protocol.ConnectionID, error) {
 				return connID, nil
 			}
 		})
 
 		AfterEach(func() {
-			generateConnectionID = origGenerateConnectionID
 			generateConnectionIDForInitial = origGenerateConnectionIDForInitial
 		})
 
@@ -119,9 +112,9 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			remoteAddrChan := make(chan string, 1)
-			newClientSession = func(
-				conn sendConn,
-				_ sessionRunner,
+			newClientConnection = func(
+				sconn sendConn,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -133,12 +126,12 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
-				remoteAddrChan <- conn.RemoteAddr().String()
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run()
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				return sess
+			) quicConn {
+				remoteAddrChan <- sconn.RemoteAddr().String()
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run()
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				return conn
 			}
 			_, err := DialAddr("localhost:17890", tlsConf, &Config{HandshakeIdleTimeout: time.Millisecond})
 			Expect(err).ToNot(HaveOccurred())
@@ -152,9 +145,9 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			hostnameChan := make(chan string, 1)
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -166,12 +159,12 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
+			) quicConn {
 				hostnameChan <- tlsConf.ServerName
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run()
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				return sess
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run()
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				return conn
 			}
 			tlsConf.ServerName = "foobar"
 			_, err := DialAddr("localhost:17890", tlsConf, nil)
@@ -185,9 +178,9 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			hostnameChan := make(chan string, 1)
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -199,12 +192,12 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
+			) quicConn {
 				hostnameChan <- tlsConf.ServerName
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				sess.EXPECT().run()
-				return sess
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				conn.EXPECT().run()
+				return conn
 			}
 			tracer.EXPECT().StartedConnection(packetConn.LocalAddr(), addr, gomock.Any(), gomock.Any())
 			_, err := Dial(
@@ -224,9 +217,9 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			run := make(chan struct{})
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				runner sessionRunner,
+				runner connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -238,14 +231,14 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
+			) quicConn {
 				Expect(enable0RTT).To(BeFalse())
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run().Do(func() { close(run) })
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run().Do(func() { close(run) })
 				ctx, cancel := context.WithCancel(context.Background())
 				cancel()
-				sess.EXPECT().HandshakeComplete().Return(ctx)
-				return sess
+				conn.EXPECT().HandshakeComplete().Return(ctx)
+				return conn
 			}
 			tracer.EXPECT().StartedConnection(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any())
 			s, err := Dial(
@@ -260,16 +253,16 @@ var _ = Describe("Client", func() {
 			Eventually(run).Should(BeClosed())
 		})
 
-		It("returns early sessions", func() {
+		It("returns early connections", func() {
 			manager := NewMockPacketHandlerManager(mockCtrl)
 			manager.EXPECT().Add(gomock.Any(), gomock.Any())
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			readyChan := make(chan struct{})
 			done := make(chan struct{})
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				runner sessionRunner,
+				runner connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -281,13 +274,13 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
+			) quicConn {
 				Expect(enable0RTT).To(BeTrue())
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run().Do(func() { <-done })
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				sess.EXPECT().earlySessionReady().Return(readyChan)
-				return sess
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run().Do(func() { <-done })
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				conn.EXPECT().earlyConnReady().Return(readyChan)
+				return conn
 			}
 
 			go func() {
@@ -315,9 +308,9 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			testErr := errors.New("early handshake error")
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -329,11 +322,11 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run().Return(testErr)
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				return sess
+			) quicConn {
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run().Return(testErr)
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				return conn
 			}
 			tracer.EXPECT().StartedConnection(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any())
 			_, err := Dial(
@@ -346,21 +339,21 @@ var _ = Describe("Client", func() {
 			Expect(err).To(MatchError(testErr))
 		})
 
-		It("closes the session when the context is canceled", func() {
+		It("closes the connection when the context is canceled", func() {
 			manager := NewMockPacketHandlerManager(mockCtrl)
 			manager.EXPECT().Add(gomock.Any(), gomock.Any())
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
-			sessionRunning := make(chan struct{})
-			defer close(sessionRunning)
-			sess := NewMockQuicSession(mockCtrl)
-			sess.EXPECT().run().Do(func() {
-				<-sessionRunning
+			connRunning := make(chan struct{})
+			defer close(connRunning)
+			conn := NewMockQuicConn(mockCtrl)
+			conn.EXPECT().run().Do(func() {
+				<-connRunning
 			})
-			sess.EXPECT().HandshakeComplete().Return(context.Background())
-			newClientSession = func(
+			conn.EXPECT().HandshakeComplete().Return(context.Background())
+			newClientConnection = func(
 				_ sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -372,8 +365,8 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
-				return sess
+			) quicConn {
+				return conn
 			}
 			ctx, cancel := context.WithCancel(context.Background())
 			dialed := make(chan struct{})
@@ -392,7 +385,7 @@ var _ = Describe("Client", func() {
 				close(dialed)
 			}()
 			Consistently(dialed).ShouldNot(BeClosed())
-			sess.EXPECT().shutdown()
+			conn.EXPECT().shutdown()
 			cancel()
 			Eventually(dialed).Should(BeClosed())
 		})
@@ -406,13 +399,13 @@ var _ = Describe("Client", func() {
 			mockMultiplexer.EXPECT().AddConn(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 			manager.EXPECT().Add(gomock.Any(), gomock.Any())
 
-			var conn sendConn
+			var sconn sendConn
 			run := make(chan struct{})
-			sessionCreated := make(chan struct{})
-			sess := NewMockQuicSession(mockCtrl)
-			newClientSession = func(
+			connCreated := make(chan struct{})
+			conn := NewMockQuicConn(mockCtrl)
+			newClientConnection = func(
 				connP sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				_ *Config,
@@ -424,15 +417,15 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				_ protocol.VersionNumber,
-			) quicSession {
-				conn = connP
-				close(sessionCreated)
-				return sess
+			) quicConn {
+				sconn = connP
+				close(connCreated)
+				return conn
 			}
-			sess.EXPECT().run().Do(func() {
+			conn.EXPECT().run().Do(func() {
 				<-run
 			})
-			sess.EXPECT().HandshakeComplete().Return(context.Background())
+			conn.EXPECT().HandshakeComplete().Return(context.Background())
 
 			done := make(chan struct{})
 			go func() {
@@ -442,10 +435,10 @@ var _ = Describe("Client", func() {
 				close(done)
 			}()
 
-			Eventually(sessionCreated).Should(BeClosed())
+			Eventually(connCreated).Should(BeClosed())
 
 			// check that the connection is not closed
-			Expect(conn.Write([]byte("foobar"))).To(Succeed())
+			Expect(sconn.Write([]byte("foobar"))).To(Succeed())
 
 			manager.EXPECT().Destroy()
 			close(run)
@@ -520,19 +513,19 @@ var _ = Describe("Client", func() {
 			})
 		})
 
-		It("creates new sessions with the right parameters", func() {
+		It("creates new connections with the right parameters", func() {
 			manager := NewMockPacketHandlerManager(mockCtrl)
 			manager.EXPECT().Add(connID, gomock.Any())
 			mockMultiplexer.EXPECT().AddConn(packetConn, gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
-			config := &Config{Versions: []protocol.VersionNumber{protocol.VersionTLS}}
+			config := &Config{Versions: []protocol.VersionNumber{protocol.VersionTLS}, ConnectionIDGenerator: &mockedConnIDGenerator{ConnID: connID}}
 			c := make(chan struct{})
 			var cconn sendConn
 			var version protocol.VersionNumber
 			var conf *Config
-			newClientSession = func(
+			newClientConnection = func(
 				connP sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				configP *Config,
@@ -544,16 +537,16 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				versionP protocol.VersionNumber,
-			) quicSession {
+			) quicConn {
 				cconn = connP
 				version = versionP
 				conf = configP
 				close(c)
 				// TODO: check connection IDs?
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().run()
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
-				return sess
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().run()
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
+				return conn
 			}
 			_, err := Dial(packetConn, addr, "localhost:1337", tlsConf, config)
 			Expect(err).ToNot(HaveOccurred())
@@ -563,16 +556,16 @@ var _ = Describe("Client", func() {
 			Expect(conf.Versions).To(Equal(config.Versions))
 		})
 
-		It("creates a new session after version negotiation", func() {
+		It("creates a new connections after version negotiation", func() {
 			manager := NewMockPacketHandlerManager(mockCtrl)
 			manager.EXPECT().Add(connID, gomock.Any()).Times(2)
 			manager.EXPECT().Destroy()
 			mockMultiplexer.EXPECT().AddConn(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(manager, nil)
 
 			var counter int
-			newClientSession = func(
+			newClientConnection = func(
 				_ sendConn,
-				_ sessionRunner,
+				_ connRunner,
 				_ protocol.ConnectionID,
 				_ protocol.ConnectionID,
 				configP *Config,
@@ -584,25 +577,26 @@ var _ = Describe("Client", func() {
 				_ uint64,
 				_ utils.Logger,
 				versionP protocol.VersionNumber,
-			) quicSession {
-				sess := NewMockQuicSession(mockCtrl)
-				sess.EXPECT().HandshakeComplete().Return(context.Background())
+			) quicConn {
+				conn := NewMockQuicConn(mockCtrl)
+				conn.EXPECT().HandshakeComplete().Return(context.Background())
 				if counter == 0 {
 					Expect(pn).To(BeZero())
 					Expect(hasNegotiatedVersion).To(BeFalse())
-					sess.EXPECT().run().Return(&errCloseForRecreating{
+					conn.EXPECT().run().Return(&errCloseForRecreating{
 						nextPacketNumber: 109,
 						nextVersion:      789,
 					})
 				} else {
 					Expect(pn).To(Equal(protocol.PacketNumber(109)))
 					Expect(hasNegotiatedVersion).To(BeTrue())
-					sess.EXPECT().run()
+					conn.EXPECT().run()
 				}
 				counter++
-				return sess
+				return conn
 			}
 
+			config := &Config{Tracer: config.Tracer, Versions: []protocol.VersionNumber{protocol.VersionTLS}, ConnectionIDGenerator: &mockedConnIDGenerator{ConnID: connID}}
 			tracer.EXPECT().StartedConnection(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any())
 			_, err := DialAddr("localhost:7890", tlsConf, config)
 			Expect(err).ToNot(HaveOccurred())
@@ -610,3 +604,15 @@ var _ = Describe("Client", func() {
 		})
 	})
 })
+
+type mockedConnIDGenerator struct {
+	ConnID protocol.ConnectionID
+}
+
+func (m *mockedConnIDGenerator) GenerateConnectionID() ([]byte, error) {
+	return m.ConnID, nil
+}
+
+func (m *mockedConnIDGenerator) ConnectionIDLen() int {
+	return m.ConnID.Len()
+}
