@@ -14,6 +14,7 @@ const version = protocol.VersionTLS
 const PrefixLen = 1
 
 // Fuzz fuzzes the QUIC header.
+//
 //go:generate go run ./cmd/corpus.go
 func Fuzz(data []byte) int {
 	if len(data) < PrefixLen {
@@ -91,8 +92,6 @@ func fuzzVNP(data []byte) int {
 	if len(versions) == 0 {
 		panic("no versions")
 	}
-	if _, err := wire.ComposeVersionNegotiation(hdr.SrcConnectionID, hdr.DestConnectionID, versions); err != nil {
-		panic(err)
-	}
+	wire.ComposeVersionNegotiation(hdr.SrcConnectionID, hdr.DestConnectionID, versions)
 	return 1
 }
