@@ -12,6 +12,7 @@ import (
 	"github.com/lucas-clemente/quic-go"
 	"github.com/lucas-clemente/quic-go/integrationtests/tools/israce"
 	"github.com/lucas-clemente/quic-go/internal/protocol"
+	"github.com/lucas-clemente/quic-go/internal/qerr"
 	"github.com/lucas-clemente/quic-go/logging"
 
 	. "github.com/onsi/ginkgo"
@@ -49,7 +50,7 @@ func (c *tokenStore) Pop(key string) *quic.ClientToken {
 }
 
 type versionNegotiationTracer struct {
-	connTracer
+	logging.NullConnectionTracer
 
 	loggedVersions                 bool
 	receivedVersionNegotiation     bool
@@ -131,14 +132,14 @@ var _ = Describe("Handshake tests", func() {
 				runServer(getTLSConfig())
 				defer server.Close()
 				clientTracer := &versionNegotiationTracer{}
-				sess, err := quic.DialAddr(
+				conn, err := quic.DialAddr(
 					fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port),
 					getTLSClientConfig(),
 					getQuicConfig(&quic.Config{Tracer: newTracer(func() logging.ConnectionTracer { return clientTracer })}),
 				)
 				Expect(err).ToNot(HaveOccurred())
-				Expect(sess.(versioner).GetVersion()).To(Equal(expectedVersion))
-				Expect(sess.CloseWithError(0, "")).To(Succeed())
+				Expect(conn.(versioner).GetVersion()).To(Equal(expectedVersion))
+				Expect(conn.CloseWithError(0, "")).To(Succeed())
 				Expect(clientTracer.chosen).To(Equal(expectedVersion))
 				Expect(clientTracer.receivedVersionNegotiation).To(BeFalse())
 				Expect(clientTracer.clientVersions).To(Equal(protocol.SupportedVersions))
@@ -159,7 +160,7 @@ var _ = Describe("Handshake tests", func() {
 				defer server.Close()
 				clientVersions := []protocol.VersionNumber{7, 8, 9, protocol.SupportedVersions[0], 10}
 				clientTracer := &versionNegotiationTracer{}
-				sess, err := quic.DialAddr(
+				conn, err := quic.DialAddr(
 					fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port),
 					getTLSClientConfig(),
 					getQuicConfig(&quic.Config{
@@ -168,8 +169,8 @@ var _ = Describe("Handshake tests", func() {
 					}),
 				)
 				Expect(err).ToNot(HaveOccurred())
-				Expect(sess.(versioner).GetVersion()).To(Equal(protocol.SupportedVersions[0]))
-				Expect(sess.CloseWithError(0, "")).To(Succeed())
+				Expect(conn.(versioner).GetVersion()).To(Equal(protocol.SupportedVersions[0]))
+				Expect(conn.CloseWithError(0, "")).To(Succeed())
 				Expect(clientTracer.chosen).To(Equal(expectedVersion))
 				Expect(clientTracer.receivedVersionNegotiation).To(BeTrue())
 				Expect(clientTracer.clientVersions).To(Equal(clientVersions))
@@ -199,28 +200,28 @@ var _ = Describe("Handshake tests", func() {
 
 				go func() {
 					defer GinkgoRecover()
-					sess, err := ln.Accept(context.Background())
+					conn, err := ln.Accept(context.Background())
 					Expect(err).ToNot(HaveOccurred())
-					str, err := sess.OpenStream()
+					str, err := conn.OpenStream()
 					Expect(err).ToNot(HaveOccurred())
 					defer str.Close()
 					_, err = str.Write(PRData)
 					Expect(err).ToNot(HaveOccurred())
 				}()
 
-				sess, err := quic.DialAddr(
+				conn, err := quic.DialAddr(
 					fmt.Sprintf("localhost:%d", ln.Addr().(*net.UDPAddr).Port),
 					getTLSClientConfig(),
 					nil,
 				)
 				Expect(err).ToNot(HaveOccurred())
-				str, err := sess.AcceptStream(context.Background())
+				str, err := conn.AcceptStream(context.Background())
 				Expect(err).ToNot(HaveOccurred())
 				data, err := io.ReadAll(str)
 				Expect(err).ToNot(HaveOccurred())
 				Expect(data).To(Equal(PRData))
-				Expect(sess.ConnectionState().TLS.CipherSuite).To(Equal(suiteID))
-				Expect(sess.CloseWithError(0, "")).To(Succeed())
+				Expect(conn.ConnectionState().TLS.CipherSuite).To(Equal(suiteID))
+				Expect(conn.CloseWithError(0, "")).To(Succeed())
 			})
 		}
 	})
@@ -280,19 +281,19 @@ var _ = Describe("Handshake tests", func() {
 					tlsConf.ClientAuth = tls.RequireAndVerifyClientCert
 					runServer(tlsConf)
 
-					sess, err := quic.DialAddr(
+					conn, err := quic.DialAddr(
 						fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port),
 						getTLSClientConfig(),
 						clientConfig,
 					)
 					// Usually, the error will occur after the client already finished the handshake.
 					// However, there's a race condition here. The server's CONNECTION_CLOSE might be
-					// received before the session is returned, so we might already get the error while dialing.
+					// received before the connection is returned, so we might already get the error while dialing.
 					if err == nil {
 						errChan := make(chan error)
 						go func() {
 							defer GinkgoRecover()
-							_, err := sess.AcceptStream(context.Background())
+							_, err := conn.AcceptStream(context.Background())
 							errChan <- err
 						}()
 						Eventually(errChan).Should(Receive(&err))
@@ -329,7 +330,7 @@ var _ = Describe("Handshake tests", func() {
 			pconn  net.PacketConn
 		)
 
-		dial := func() (quic.Session, error) {
+		dial := func() (quic.Connection, error) {
 			remoteAddr := fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port)
 			raddr, err := net.ResolveUDPAddr("udp", remoteAddr)
 			Expect(err).ToNot(HaveOccurred())
@@ -343,12 +344,6 @@ var _ = Describe("Handshake tests", func() {
 		}
 
 		BeforeEach(func() {
-			serverConfig.AcceptToken = func(addr net.Addr, token *quic.Token) bool {
-				if token != nil {
-					Expect(token.IsRetryToken).To(BeFalse())
-				}
-				return true
-			}
 			var err error
 			// start the server, but don't call Accept
 			server, err = quic.ListenAddr("localhost:0", getTLSConfig(), serverConfig)
@@ -368,11 +363,11 @@ var _ = Describe("Handshake tests", func() {
 
 		It("rejects new connection attempts if connections don't get accepted", func() {
 			for i := 0; i < protocol.MaxAcceptQueueSize; i++ {
-				sess, err := dial()
+				conn, err := dial()
 				Expect(err).ToNot(HaveOccurred())
-				defer sess.CloseWithError(0, "")
+				defer conn.CloseWithError(0, "")
 			}
-			time.Sleep(25 * time.Millisecond) // wait a bit for the sessions to be queued
+			time.Sleep(25 * time.Millisecond) // wait a bit for the connection to be queued
 
 			_, err := dial()
 			Expect(err).To(HaveOccurred())
@@ -380,14 +375,14 @@ var _ = Describe("Handshake tests", func() {
 			Expect(errors.As(err, &transportErr)).To(BeTrue())
 			Expect(transportErr.ErrorCode).To(Equal(quic.ConnectionRefused))
 
-			// now accept one session, freeing one spot in the queue
+			// now accept one connection, freeing one spot in the queue
 			_, err = server.Accept(context.Background())
 			Expect(err).ToNot(HaveOccurred())
 			// dial again, and expect that this dial succeeds
-			sess, err := dial()
+			conn, err := dial()
 			Expect(err).ToNot(HaveOccurred())
-			defer sess.CloseWithError(0, "")
-			time.Sleep(25 * time.Millisecond) // wait a bit for the session to be queued
+			defer conn.CloseWithError(0, "")
+			time.Sleep(25 * time.Millisecond) // wait a bit for the connection to be queued
 
 			_, err = dial()
 			Expect(err).To(HaveOccurred())
@@ -396,15 +391,15 @@ var _ = Describe("Handshake tests", func() {
 		})
 
 		It("removes closed connections from the accept queue", func() {
-			firstSess, err := dial()
+			firstConn, err := dial()
 			Expect(err).ToNot(HaveOccurred())
 
 			for i := 1; i < protocol.MaxAcceptQueueSize; i++ {
-				sess, err := dial()
+				conn, err := dial()
 				Expect(err).ToNot(HaveOccurred())
-				defer sess.CloseWithError(0, "")
+				defer conn.CloseWithError(0, "")
 			}
-			time.Sleep(scaleDuration(20 * time.Millisecond)) // wait a bit for the sessions to be queued
+			time.Sleep(scaleDuration(20 * time.Millisecond)) // wait a bit for the connection to be queued
 
 			_, err = dial()
 			Expect(err).To(HaveOccurred())
@@ -412,16 +407,16 @@ var _ = Describe("Handshake tests", func() {
 			Expect(errors.As(err, &transportErr)).To(BeTrue())
 			Expect(transportErr.ErrorCode).To(Equal(quic.ConnectionRefused))
 
-			// Now close the one of the session that are waiting to be accepted.
+			// Now close the one of the connection that are waiting to be accepted.
 			// This should free one spot in the queue.
-			Expect(firstSess.CloseWithError(0, ""))
-			Eventually(firstSess.Context().Done()).Should(BeClosed())
+			Expect(firstConn.CloseWithError(0, ""))
+			Eventually(firstConn.Context().Done()).Should(BeClosed())
 			time.Sleep(scaleDuration(20 * time.Millisecond))
 
 			// dial again, and expect that this dial succeeds
 			_, err = dial()
 			Expect(err).ToNot(HaveOccurred())
-			time.Sleep(scaleDuration(20 * time.Millisecond)) // wait a bit for the session to be queued
+			time.Sleep(scaleDuration(20 * time.Millisecond)) // wait a bit for the connection to be queued
 
 			_, err = dial()
 			Expect(err).To(HaveOccurred())
@@ -438,21 +433,21 @@ var _ = Describe("Handshake tests", func() {
 			done := make(chan struct{})
 			go func() {
 				defer GinkgoRecover()
-				sess, err := ln.Accept(context.Background())
+				conn, err := ln.Accept(context.Background())
 				Expect(err).ToNot(HaveOccurred())
-				cs := sess.ConnectionState()
+				cs := conn.ConnectionState()
 				Expect(cs.TLS.NegotiatedProtocol).To(Equal(alpn))
 				close(done)
 			}()
 
-			sess, err := quic.DialAddr(
+			conn, err := quic.DialAddr(
 				fmt.Sprintf("localhost:%d", ln.Addr().(*net.UDPAddr).Port),
 				getTLSClientConfig(),
 				nil,
 			)
 			Expect(err).ToNot(HaveOccurred())
-			defer sess.CloseWithError(0, "")
-			cs := sess.ConnectionState()
+			defer conn.CloseWithError(0, "")
+			cs := conn.ConnectionState()
 			Expect(cs.TLS.NegotiatedProtocol).To(Equal(alpn))
 			Eventually(done).Should(BeClosed())
 			Expect(ln.Close()).To(Succeed())
@@ -478,18 +473,10 @@ var _ = Describe("Handshake tests", func() {
 
 	Context("using tokens", func() {
 		It("uses tokens provided in NEW_TOKEN frames", func() {
-			tokenChan := make(chan *quic.Token, 100)
-			serverConfig.AcceptToken = func(addr net.Addr, token *quic.Token) bool {
-				if token != nil && !token.IsRetryToken {
-					tokenChan <- token
-				}
-				return true
-			}
-
 			server, err := quic.ListenAddr("localhost:0", getTLSConfig(), serverConfig)
 			Expect(err).ToNot(HaveOccurred())
 
-			// dial the first session and receive the token
+			// dial the first connection and receive the token
 			go func() {
 				defer GinkgoRecover()
 				_, err := server.Accept(context.Background())
@@ -500,7 +487,7 @@ var _ = Describe("Handshake tests", func() {
 			puts := make(chan string, 100)
 			tokenStore := newTokenStore(gets, puts)
 			quicConf := getQuicConfig(&quic.Config{TokenStore: tokenStore})
-			sess, err := quic.DialAddr(
+			conn, err := quic.DialAddr(
 				fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port),
 				getTLSClientConfig(),
 				quicConf,
@@ -508,11 +495,10 @@ var _ = Describe("Handshake tests", func() {
 			Expect(err).ToNot(HaveOccurred())
 			Expect(gets).To(Receive())
 			Eventually(puts).Should(Receive())
-			Expect(tokenChan).ToNot(Receive())
-			// received a token. Close this session.
-			Expect(sess.CloseWithError(0, "")).To(Succeed())
+			// received a token. Close this connection.
+			Expect(conn.CloseWithError(0, "")).To(Succeed())
 
-			// dial the second session and verify that the token was used
+			// dial the second connection and verify that the token was used
 			done := make(chan struct{})
 			go func() {
 				defer GinkgoRecover()
@@ -520,25 +506,21 @@ var _ = Describe("Handshake tests", func() {
 				_, err := server.Accept(context.Background())
 				Expect(err).ToNot(HaveOccurred())
 			}()
-			sess, err = quic.DialAddr(
+			conn, err = quic.DialAddr(
 				fmt.Sprintf("localhost:%d", server.Addr().(*net.UDPAddr).Port),
 				getTLSClientConfig(),
 				quicConf,
 			)
 			Expect(err).ToNot(HaveOccurred())
-			defer sess.CloseWithError(0, "")
+			defer conn.CloseWithError(0, "")
 			Expect(gets).To(Receive())
-			Expect(tokenChan).To(Receive())
 
 			Eventually(done).Should(BeClosed())
 		})
 
 		It("rejects invalid Retry token with the INVALID_TOKEN error", func() {
-			tokenChan := make(chan *quic.Token, 10)
-			serverConfig.AcceptToken = func(addr net.Addr, token *quic.Token) bool {
-				tokenChan <- token
-				return false
-			}
+			serverConfig.RequireAddressValidation = func(net.Addr) bool { return true }
+			serverConfig.MaxRetryTokenAge = time.Nanosecond
 
 			server, err := quic.ListenAddr("localhost:0", getTLSConfig(), serverConfig)
 			Expect(err).ToNot(HaveOccurred())
@@ -553,18 +535,39 @@ var _ = Describe("Handshake tests", func() {
 			var transportErr *quic.TransportError
 			Expect(errors.As(err, &transportErr)).To(BeTrue())
 			Expect(transportErr.ErrorCode).To(Equal(quic.InvalidToken))
-			// Receiving a Retry might lead the client to measure a very small RTT.
-			// Then, it sometimes would retransmit the ClientHello before receiving the ServerHello.
-			Expect(len(tokenChan)).To(BeNumerically(">=", 2))
-			token := <-tokenChan
-			Expect(token).To(BeNil())
-			token = <-tokenChan
-			Expect(token).ToNot(BeNil())
-			// If the ClientHello was retransmitted, make sure that it contained the same Retry token.
-			for i := 2; i < len(tokenChan); i++ {
-				Expect(<-tokenChan).To(Equal(token))
-			}
-			Expect(token.IsRetryToken).To(BeTrue())
 		})
 	})
+
+	It("doesn't send any packets when generating the ClientHello fails", func() {
+		ln, err := net.ListenUDP("udp", nil)
+		Expect(err).ToNot(HaveOccurred())
+		done := make(chan struct{})
+		packetChan := make(chan struct{})
+		go func() {
+			defer GinkgoRecover()
+			defer close(done)
+			for {
+				_, _, err := ln.ReadFromUDP(make([]byte, protocol.MaxPacketBufferSize))
+				if err != nil {
+					return
+				}
+				packetChan <- struct{}{}
+			}
+		}()
+
+		tlsConf := getTLSClientConfig()
+		tlsConf.NextProtos = []string{""}
+		_, err = quic.DialAddr(
+			fmt.Sprintf("localhost:%d", ln.LocalAddr().(*net.UDPAddr).Port),
+			tlsConf,
+			nil,
+		)
+		Expect(err).To(MatchError(&qerr.TransportError{
+			ErrorCode:    qerr.InternalError,
+			ErrorMessage: "tls: invalid NextProtos value",
+		}))
+		Consistently(packetChan).ShouldNot(Receive())
+		ln.Close()
+		Eventually(done).Should(BeClosed())
+	})
 })
