@@ -3,7 +3,7 @@ package testdata
 import (
 	"crypto/tls"
 	"crypto/x509"
-	"io/ioutil"
+	"os"
 	"path"
 	"runtime"
 )
@@ -38,7 +38,7 @@ func GetTLSConfig() *tls.Config {
 // AddRootCA adds the root CA certificate to a cert pool
 func AddRootCA(certPool *x509.CertPool) {
 	caCertPath := path.Join(certPath, "ca.pem")
-	caCertRaw, err := ioutil.ReadFile(caCertPath)
+	caCertRaw, err := os.ReadFile(caCertPath)
 	if err != nil {
 		panic(err)
 	}
