@@ -2,7 +2,7 @@ package testdata
 
 import (
 	"crypto/tls"
-	"io/ioutil"
+	"io"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -24,7 +24,7 @@ var _ = Describe("certificates", func() {
 
 		conn, err := tls.Dial("tcp", "localhost:4433", &tls.Config{RootCAs: GetRootCA()})
 		Expect(err).ToNot(HaveOccurred())
-		data, err := ioutil.ReadAll(conn)
+		data, err := io.ReadAll(conn)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(string(data)).To(Equal("foobar"))
 	})
