@@ -7,6 +7,8 @@ import (
 
 	"github.com/lucas-clemente/quic-go/internal/protocol"
 	"github.com/lucas-clemente/quic-go/internal/utils"
+	"github.com/lucas-clemente/quic-go/quicvarint"
+
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 )
@@ -32,7 +34,7 @@ var _ = Describe("Header", func() {
 						DestConnectionID: protocol.ConnectionID{0xde, 0xad, 0xbe, 0xef, 0xca, 0xfe},
 						SrcConnectionID:  protocol.ConnectionID{0xde, 0xca, 0xfb, 0xad, 0x0, 0x0, 0x13, 0x37},
 						Version:          0x1020304,
-						Length:           protocol.MaxPacketSizeIPv4,
+						Length:           protocol.InitialPacketSizeIPv4,
 					},
 					PacketNumber:    0xdecaf,
 					PacketNumberLen: protocol.PacketNumberLen3,
@@ -45,8 +47,8 @@ var _ = Describe("Header", func() {
 					0x8,                                          // src connection ID length
 					0xde, 0xca, 0xfb, 0xad, 0x0, 0x0, 0x13, 0x37, // source connection ID
 				}
-				expected = append(expected, encodeVarInt(protocol.MaxPacketSizeIPv4)...) // length
-				expected = append(expected, []byte{0xd, 0xec, 0xaf}...)                  // packet number
+				expected = append(expected, encodeVarInt(protocol.InitialPacketSizeIPv4)...) // length
+				expected = append(expected, []byte{0xd, 0xec, 0xaf}...)                      // packet number
 				Expect(buf.Bytes()).To(Equal(expected))
 			})
 
@@ -93,6 +95,7 @@ var _ = Describe("Header", func() {
 					PacketNumber:    0xdecafbad,
 					PacketNumberLen: protocol.PacketNumberLen4,
 				}).Write(buf, versionIETFHeader)).To(Succeed())
+				Expect(buf.Bytes()[0]>>4&0b11 == 0)
 				expectedSubstring := append(encodeVarInt(uint64(len(token))), token...)
 				Expect(buf.Bytes()).To(ContainSubstring(string(expectedSubstring)))
 			})
@@ -109,7 +112,7 @@ var _ = Describe("Header", func() {
 					PacketNumberLen: protocol.PacketNumberLen4,
 				}).Write(buf, versionIETFHeader)).To(Succeed())
 				b := &bytes.Buffer{}
-				utils.WriteVarIntWithLen(b, 37, 2)
+				quicvarint.WriteWithLen(b, 37, 2)
 				Expect(buf.Bytes()[buf.Len()-6 : buf.Len()-4]).To(Equal(b.Bytes()))
 			})
 
@@ -117,21 +120,76 @@ var _ = Describe("Header", func() {
 				token := []byte("Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.")
 				Expect((&ExtendedHeader{Header: Header{
 					IsLongHeader: true,
-					Version:      0x1020304,
+					Version:      protocol.Version1,
 					Type:         protocol.PacketTypeRetry,
 					Token:        token,
 				}}).Write(buf, versionIETFHeader)).To(Succeed())
-				expected := []byte{
-					0xc0 | 0x3<<4,
-					0x1, 0x2, 0x3, 0x4, // version number
-					0x0, // dest connection ID length
-					0x0, // src connection ID length
-				}
+				expected := []byte{0xc0 | 0b11<<4}
+				expected = appendVersion(expected, protocol.Version1)
+				expected = append(expected, 0x0) // dest connection ID length
+				expected = append(expected, 0x0) // src connection ID length
 				expected = append(expected, token...)
 				Expect(buf.Bytes()).To(Equal(expected))
 			})
 		})
 
+		Context("long header, version 2", func() {
+			It("writes an Initial", func() {
+				Expect((&ExtendedHeader{
+					Header: Header{
+						IsLongHeader: true,
+						Version:      protocol.Version2,
+						Type:         protocol.PacketTypeInitial,
+					},
+					PacketNumber:    0xdecafbad,
+					PacketNumberLen: protocol.PacketNumberLen4,
+				}).Write(buf, protocol.Version2)).To(Succeed())
+				Expect(buf.Bytes()[0]>>4&0b11 == 0b01)
+			})
+
+			It("writes a Retry packet", func() {
+				token := []byte("Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.")
+				Expect((&ExtendedHeader{Header: Header{
+					IsLongHeader: true,
+					Version:      protocol.Version2,
+					Type:         protocol.PacketTypeRetry,
+					Token:        token,
+				}}).Write(buf, versionIETFHeader)).To(Succeed())
+				expected := []byte{0xc0 | 0b11<<4}
+				expected = appendVersion(expected, protocol.Version2)
+				expected = append(expected, 0x0) // dest connection ID length
+				expected = append(expected, 0x0) // src connection ID length
+				expected = append(expected, token...)
+				Expect(buf.Bytes()).To(Equal(expected))
+			})
+
+			It("writes a Handshake Packet", func() {
+				Expect((&ExtendedHeader{
+					Header: Header{
+						IsLongHeader: true,
+						Version:      protocol.Version2,
+						Type:         protocol.PacketTypeHandshake,
+					},
+					PacketNumber:    0xdecafbad,
+					PacketNumberLen: protocol.PacketNumberLen4,
+				}).Write(buf, protocol.Version2)).To(Succeed())
+				Expect(buf.Bytes()[0]>>4&0b11 == 0b11)
+			})
+
+			It("writes a 0-RTT Packet", func() {
+				Expect((&ExtendedHeader{
+					Header: Header{
+						IsLongHeader: true,
+						Version:      protocol.Version2,
+						Type:         protocol.PacketType0RTT,
+					},
+					PacketNumber:    0xdecafbad,
+					PacketNumberLen: protocol.PacketNumberLen4,
+				}).Write(buf, protocol.Version2)).To(Succeed())
+				Expect(buf.Bytes()[0]>>4&0b11 == 0b10)
+			})
+		})
+
 		Context("short header", func() {
 			It("writes a header with connection ID", func() {
 				Expect((&ExtendedHeader{
@@ -358,7 +416,7 @@ var _ = Describe("Header", func() {
 				PacketNumber:    1337,
 				PacketNumberLen: protocol.PacketNumberLen2,
 			}).Log(logger)
-			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Handshake, DestConnectionID: 0xdeadbeefcafe1337, SrcConnectionID: 0xdecafbad13371337, PacketNumber: 1337, PacketNumberLen: 2, Length: 54321, Version: 0xfeed}"))
+			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Handshake, DestConnectionID: deadbeefcafe1337, SrcConnectionID: decafbad13371337, PacketNumber: 1337, PacketNumberLen: 2, Length: 54321, Version: 0xfeed}"))
 		})
 
 		It("logs Initial Packets with a Token", func() {
@@ -375,7 +433,7 @@ var _ = Describe("Header", func() {
 				PacketNumber:    42,
 				PacketNumberLen: protocol.PacketNumberLen2,
 			}).Log(logger)
-			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Initial, DestConnectionID: 0xcafe1337, SrcConnectionID: 0xdecafbad, Token: 0xdeadbeef, PacketNumber: 42, PacketNumberLen: 2, Length: 100, Version: 0xfeed}"))
+			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Initial, DestConnectionID: cafe1337, SrcConnectionID: decafbad, Token: 0xdeadbeef, PacketNumber: 42, PacketNumberLen: 2, Length: 100, Version: 0xfeed}"))
 		})
 
 		It("logs Initial packets without a Token", func() {
@@ -391,7 +449,7 @@ var _ = Describe("Header", func() {
 				PacketNumber:    42,
 				PacketNumberLen: protocol.PacketNumberLen2,
 			}).Log(logger)
-			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Initial, DestConnectionID: 0xcafe1337, SrcConnectionID: 0xdecafbad, Token: (empty), PacketNumber: 42, PacketNumberLen: 2, Length: 100, Version: 0xfeed}"))
+			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Initial, DestConnectionID: cafe1337, SrcConnectionID: decafbad, Token: (empty), PacketNumber: 42, PacketNumberLen: 2, Length: 100, Version: 0xfeed}"))
 		})
 
 		It("logs Retry packets with a Token", func() {
@@ -405,7 +463,7 @@ var _ = Describe("Header", func() {
 					Version:          0xfeed,
 				},
 			}).Log(logger)
-			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Retry, DestConnectionID: 0xcafe1337, SrcConnectionID: 0xdecafbad, Token: 0x123456, Version: 0xfeed}"))
+			Expect(buf.String()).To(ContainSubstring("Long Header{Type: Retry, DestConnectionID: cafe1337, SrcConnectionID: decafbad, Token: 0x123456, Version: 0xfeed}"))
 		})
 
 		It("logs Short Headers containing a connection ID", func() {
@@ -417,7 +475,7 @@ var _ = Describe("Header", func() {
 				PacketNumber:    1337,
 				PacketNumberLen: 4,
 			}).Log(logger)
-			Expect(buf.String()).To(ContainSubstring("Short Header{DestConnectionID: 0xdeadbeefcafe1337, PacketNumber: 1337, PacketNumberLen: 4, KeyPhase: 1}"))
+			Expect(buf.String()).To(ContainSubstring("Short Header{DestConnectionID: deadbeefcafe1337, PacketNumber: 1337, PacketNumberLen: 4, KeyPhase: 1}"))
 		})
 	})
 })
