@@ -11,13 +11,6 @@ import (
 )
 
 var _ = Describe("Header Parsing", func() {
-	appendVersion := func(data []byte, v protocol.VersionNumber) []byte {
-		offset := len(data)
-		data = append(data, []byte{0, 0, 0, 0}...)
-		binary.BigEndian.PutUint32(data[offset:], uint32(v))
-		return data
-	}
-
 	Context("Parsing the Connection ID", func() {
 		It("parses the connection ID of a long header packet", func() {
 			buf := &bytes.Buffer{}
@@ -27,10 +20,10 @@ var _ = Describe("Header Parsing", func() {
 					Type:             protocol.PacketTypeHandshake,
 					DestConnectionID: protocol.ConnectionID{0xde, 0xca, 0xfb, 0xad},
 					SrcConnectionID:  protocol.ConnectionID{1, 2, 3, 4, 5, 6},
-					Version:          versionIETFFrames,
+					Version:          protocol.Version1,
 				},
 				PacketNumberLen: 2,
-			}).Write(buf, versionIETFFrames)).To(Succeed())
+			}).Write(buf, protocol.Version1)).To(Succeed())
 			connID, err := ParseConnectionID(buf.Bytes(), 8)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(connID).To(Equal(protocol.ConnectionID{0xde, 0xca, 0xfb, 0xad}))
@@ -43,7 +36,7 @@ var _ = Describe("Header Parsing", func() {
 					DestConnectionID: protocol.ConnectionID{0xde, 0xca, 0xfb, 0xad},
 				},
 				PacketNumberLen: 2,
-			}).Write(buf, versionIETFFrames)).To(Succeed())
+			}).Write(buf, protocol.Version1)).To(Succeed())
 			buf.Write([]byte("foobar"))
 			connID, err := ParseConnectionID(buf.Bytes(), 4)
 			Expect(err).ToNot(HaveOccurred())
@@ -57,7 +50,7 @@ var _ = Describe("Header Parsing", func() {
 					DestConnectionID: protocol.ConnectionID{1, 2, 3, 4, 5, 6, 7, 8},
 				},
 				PacketNumberLen: 2,
-			}).Write(buf, versionIETFFrames)).To(Succeed())
+			}).Write(buf, protocol.Version1)).To(Succeed())
 			data := buf.Bytes()[:buf.Len()-2] // cut the packet number
 			_, err := ParseConnectionID(data, 8)
 			Expect(err).ToNot(HaveOccurred())
@@ -77,10 +70,10 @@ var _ = Describe("Header Parsing", func() {
 					Type:             protocol.PacketTypeHandshake,
 					DestConnectionID: protocol.ConnectionID{0xde, 0xca, 0xfb, 0xad, 0x13, 0x37},
 					SrcConnectionID:  protocol.ConnectionID{1, 2, 3, 4, 5, 6, 8, 9},
-					Version:          versionIETFFrames,
+					Version:          protocol.Version1,
 				},
 				PacketNumberLen: 2,
-			}).Write(buf, versionIETFFrames)).To(Succeed())
+			}).Write(buf, protocol.Version1)).To(Succeed())
 			data := buf.Bytes()[:buf.Len()-2] // cut the packet number
 			_, err := ParseConnectionID(data, 8)
 			Expect(err).ToNot(HaveOccurred())
@@ -93,6 +86,32 @@ var _ = Describe("Header Parsing", func() {
 		})
 	})
 
+	Context("identifying 0-RTT packets", func() {
+		It("recognizes 0-RTT packets, for QUIC v1", func() {
+			zeroRTTHeader := make([]byte, 5)
+			zeroRTTHeader[0] = 0x80 | 0b01<<4
+			binary.BigEndian.PutUint32(zeroRTTHeader[1:], uint32(protocol.Version1))
+
+			Expect(Is0RTTPacket(zeroRTTHeader)).To(BeTrue())
+			Expect(Is0RTTPacket(zeroRTTHeader[:4])).To(BeFalse())                           // too short
+			Expect(Is0RTTPacket([]byte{zeroRTTHeader[0], 1, 2, 3, 4})).To(BeFalse())        // unknown version
+			Expect(Is0RTTPacket([]byte{zeroRTTHeader[0] | 0x80, 1, 2, 3, 4})).To(BeFalse()) // short header
+			Expect(Is0RTTPacket(append(zeroRTTHeader, []byte("foobar")...))).To(BeTrue())
+		})
+
+		It("recognizes 0-RTT packets, for QUIC v2", func() {
+			zeroRTTHeader := make([]byte, 5)
+			zeroRTTHeader[0] = 0x80 | 0b10<<4
+			binary.BigEndian.PutUint32(zeroRTTHeader[1:], uint32(protocol.Version2))
+
+			Expect(Is0RTTPacket(zeroRTTHeader)).To(BeTrue())
+			Expect(Is0RTTPacket(zeroRTTHeader[:4])).To(BeFalse())                           // too short
+			Expect(Is0RTTPacket([]byte{zeroRTTHeader[0], 1, 2, 3, 4})).To(BeFalse())        // unknown version
+			Expect(Is0RTTPacket([]byte{zeroRTTHeader[0] | 0x80, 1, 2, 3, 4})).To(BeFalse()) // short header
+			Expect(Is0RTTPacket(append(zeroRTTHeader, []byte("foobar")...))).To(BeTrue())
+		})
+	})
+
 	Context("Identifying Version Negotiation Packets", func() {
 		It("identifies version negotiation packets", func() {
 			Expect(IsVersionNegotiationPacket([]byte{0x80 | 0x56, 0, 0, 0, 0})).To(BeTrue())
@@ -116,7 +135,7 @@ var _ = Describe("Header Parsing", func() {
 			destConnID := protocol.ConnectionID{9, 8, 7, 6, 5, 4, 3, 2, 1}
 			srcConnID := protocol.ConnectionID{0xde, 0xad, 0xbe, 0xef}
 			data := []byte{0xc0 ^ 0x3}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, 0x9) // dest conn id length
 			data = append(data, destConnID...)
 			data = append(data, 0x4) // src conn id length
@@ -138,10 +157,10 @@ var _ = Describe("Header Parsing", func() {
 			Expect(hdr.Type).To(Equal(protocol.PacketTypeInitial))
 			Expect(hdr.Token).To(Equal([]byte("foobar")))
 			Expect(hdr.Length).To(Equal(protocol.ByteCount(10)))
-			Expect(hdr.Version).To(Equal(versionIETFFrames))
+			Expect(hdr.Version).To(Equal(protocol.Version1))
 			Expect(rest).To(BeEmpty())
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0xbeef)))
 			Expect(extHdr.PacketNumberLen).To(Equal(protocol.PacketNumberLen4))
@@ -182,7 +201,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("parses a Long Header without a destination connection ID", func() {
 			data := []byte{0xc0 ^ 0x1<<4}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, 0x0)                               // dest conn ID len
 			data = append(data, 0x4)                               // src conn ID len
 			data = append(data, []byte{0xde, 0xad, 0xbe, 0xef}...) // source connection ID
@@ -197,7 +216,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("parses a Long Header without a source connection ID", func() {
 			data := []byte{0xc0 ^ 0x2<<4}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, 0xa)                                      // dest conn ID len
 			data = append(data, []byte{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}...) // dest connection ID
 			data = append(data, 0x0)                                      // src conn ID len
@@ -211,7 +230,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("parses a Long Header with a 2 byte packet number", func() {
 			data := []byte{0xc0 ^ 0x1}
-			data = appendVersion(data, versionIETFFrames) // version number
+			data = appendVersion(data, protocol.Version1) // version number
 			data = append(data, []byte{0x0, 0x0}...)      // connection ID lengths
 			data = append(data, encodeVarInt(0)...)       // token length
 			data = append(data, encodeVarInt(0)...)       // length
@@ -220,16 +239,16 @@ var _ = Describe("Header Parsing", func() {
 			hdr, _, _, err := ParsePacket(data, 0)
 			Expect(err).ToNot(HaveOccurred())
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0x123)))
 			Expect(extHdr.PacketNumberLen).To(Equal(protocol.PacketNumberLen2))
 			Expect(b.Len()).To(BeZero())
 		})
 
-		It("parses a Retry packet", func() {
-			data := []byte{0xc0 | 0x3<<4 | (10 - 3) /* connection ID length */}
-			data = appendVersion(data, versionIETFFrames)
+		It("parses a Retry packet, for QUIC v1", func() {
+			data := []byte{0xc0 | 0b11<<4 | (10 - 3) /* connection ID length */}
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, []byte{6}...)                             // dest conn ID len
 			data = append(data, []byte{6, 5, 4, 3, 2, 1}...)              // dest conn ID
 			data = append(data, []byte{10}...)                            // src conn ID len
@@ -239,6 +258,27 @@ var _ = Describe("Header Parsing", func() {
 			hdr, pdata, rest, err := ParsePacket(data, 0)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(hdr.Type).To(Equal(protocol.PacketTypeRetry))
+			Expect(hdr.Version).To(Equal(protocol.Version1))
+			Expect(hdr.DestConnectionID).To(Equal(protocol.ConnectionID{6, 5, 4, 3, 2, 1}))
+			Expect(hdr.SrcConnectionID).To(Equal(protocol.ConnectionID{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}))
+			Expect(hdr.Token).To(Equal([]byte("foobar")))
+			Expect(pdata).To(Equal(data))
+			Expect(rest).To(BeEmpty())
+		})
+
+		It("parses a Retry packet, for QUIC v2", func() {
+			data := []byte{0xc0 | 0b00<<4 | (10 - 3) /* connection ID length */}
+			data = appendVersion(data, protocol.Version2)
+			data = append(data, []byte{6}...)                             // dest conn ID len
+			data = append(data, []byte{6, 5, 4, 3, 2, 1}...)              // dest conn ID
+			data = append(data, []byte{10}...)                            // src conn ID len
+			data = append(data, []byte{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}...) // source connection ID
+			data = append(data, []byte{'f', 'o', 'o', 'b', 'a', 'r'}...)  // token
+			data = append(data, []byte{16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1}...)
+			hdr, pdata, rest, err := ParsePacket(data, 0)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(hdr.Type).To(Equal(protocol.PacketTypeRetry))
+			Expect(hdr.Version).To(Equal(protocol.Version2))
 			Expect(hdr.DestConnectionID).To(Equal(protocol.ConnectionID{6, 5, 4, 3, 2, 1}))
 			Expect(hdr.SrcConnectionID).To(Equal(protocol.ConnectionID{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}))
 			Expect(hdr.Token).To(Equal([]byte("foobar")))
@@ -248,7 +288,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("errors if the Retry packet is too short for the integrity tag", func() {
 			data := []byte{0xc0 | 0x3<<4 | (10 - 3) /* connection ID length */}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, []byte{0, 0}...)                         // conn ID lens
 			data = append(data, []byte{'f', 'o', 'o', 'b', 'a', 'r'}...) // token
 			data = append(data, []byte{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}...)
@@ -259,7 +299,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("errors if the token length is too large", func() {
 			data := []byte{0xc0 ^ 0x1}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, 0x0)                   // connection ID lengths
 			data = append(data, encodeVarInt(4)...)    // token length: 4 bytes (1 byte too long)
 			data = append(data, encodeVarInt(0x42)...) // length, 1 byte
@@ -271,14 +311,14 @@ var _ = Describe("Header Parsing", func() {
 
 		It("errors if the 5th or 6th bit are set", func() {
 			data := []byte{0xc0 | 0x2<<4 | 0x8 /* set the 5th bit */ | 0x1 /* 2 byte packet number */}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, []byte{0x0, 0x0}...)   // connection ID lengths
 			data = append(data, encodeVarInt(2)...)    // length
 			data = append(data, []byte{0x12, 0x34}...) // packet number
 			hdr, _, _, err := ParsePacket(data, 0)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(hdr.Type).To(Equal(protocol.PacketTypeHandshake))
-			extHdr, err := hdr.ParseExtended(bytes.NewReader(data), versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(bytes.NewReader(data), protocol.Version1)
 			Expect(err).To(MatchError(ErrInvalidReservedBits))
 			Expect(extHdr).ToNot(BeNil())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0x1234)))
@@ -286,7 +326,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("errors on EOF, when parsing the header", func() {
 			data := []byte{0xc0 ^ 0x2<<4}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, 0x8)                                                       // dest conn ID len
 			data = append(data, []byte{0xde, 0xad, 0xbe, 0xef, 0xca, 0xfe, 0x13, 0x37}...) // dest conn ID
 			data = append(data, 0x8)                                                       // src conn ID len
@@ -299,7 +339,7 @@ var _ = Describe("Header Parsing", func() {
 
 		It("errors on EOF, when parsing the extended header", func() {
 			data := []byte{0xc0 | 0x2<<4 | 0x3}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, []byte{0x0, 0x0}...) // connection ID lengths
 			data = append(data, encodeVarInt(0)...)  // length
 			hdrLen := len(data)
@@ -309,14 +349,14 @@ var _ = Describe("Header Parsing", func() {
 				hdr, _, _, err := ParsePacket(data, 0)
 				Expect(err).ToNot(HaveOccurred())
 				b := bytes.NewReader(data)
-				_, err = hdr.ParseExtended(b, versionIETFFrames)
+				_, err = hdr.ParseExtended(b, protocol.Version1)
 				Expect(err).To(Equal(io.EOF))
 			}
 		})
 
 		It("errors on EOF, for a Retry packet", func() {
 			data := []byte{0xc0 ^ 0x3<<4}
-			data = appendVersion(data, versionIETFFrames)
+			data = appendVersion(data, protocol.Version1)
 			data = append(data, []byte{0x0, 0x0}...)                      // connection ID lengths
 			data = append(data, 0xa)                                      // Orig Destination Connection ID length
 			data = append(data, []byte{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}...) // source connection ID
@@ -326,7 +366,7 @@ var _ = Describe("Header Parsing", func() {
 				hdr, _, _, err := ParsePacket(data, 0)
 				Expect(err).ToNot(HaveOccurred())
 				b := bytes.NewReader(data)
-				_, err = hdr.ParseExtended(b, versionIETFFrames)
+				_, err = hdr.ParseExtended(b, protocol.Version1)
 				Expect(err).To(Equal(io.EOF))
 			}
 		})
@@ -339,13 +379,13 @@ var _ = Describe("Header Parsing", func() {
 					Type:             protocol.PacketTypeInitial,
 					DestConnectionID: protocol.ConnectionID{1, 2, 3, 4},
 					Length:           2 + 6,
-					Version:          versionIETFFrames,
+					Version:          protocol.Version1,
 				}
 				Expect((&ExtendedHeader{
 					Header:          hdr,
 					PacketNumber:    0x1337,
 					PacketNumberLen: 2,
-				}).Write(buf, versionIETFFrames)).To(Succeed())
+				}).Write(buf, protocol.Version1)).To(Succeed())
 				hdrRaw := append([]byte{}, buf.Bytes()...)
 				buf.Write([]byte("foobar")) // payload of the first packet
 				buf.Write([]byte("raboof")) // second packet
@@ -365,11 +405,11 @@ var _ = Describe("Header Parsing", func() {
 						Type:             protocol.PacketTypeInitial,
 						DestConnectionID: protocol.ConnectionID{1, 2, 3, 4},
 						Length:           3,
-						Version:          versionIETFFrames,
+						Version:          protocol.Version1,
 					},
 					PacketNumber:    0x1337,
 					PacketNumberLen: 2,
-				}).Write(buf, versionIETFFrames)).To(Succeed())
+				}).Write(buf, protocol.Version1)).To(Succeed())
 				_, _, _, err := ParsePacket(buf.Bytes(), 4)
 				Expect(err).To(HaveOccurred())
 				Expect(err.Error()).To(ContainSubstring("packet length (2 bytes) is smaller than the expected length (3 bytes)"))
@@ -383,11 +423,11 @@ var _ = Describe("Header Parsing", func() {
 						Type:             protocol.PacketTypeInitial,
 						DestConnectionID: protocol.ConnectionID{1, 2, 3, 4},
 						Length:           1000,
-						Version:          versionIETFFrames,
+						Version:          protocol.Version1,
 					},
 					PacketNumber:    0x1337,
 					PacketNumberLen: 2,
-				}).Write(buf, versionIETFFrames)).To(Succeed())
+				}).Write(buf, protocol.Version1)).To(Succeed())
 				buf.Write(make([]byte, 500-2 /* for packet number length */))
 				_, _, _, err := ParsePacket(buf.Bytes(), 4)
 				Expect(err).To(MatchError("packet length (500 bytes) is smaller than the expected length (1000 bytes)"))
@@ -407,7 +447,7 @@ var _ = Describe("Header Parsing", func() {
 			Expect(hdr.IsLongHeader).To(BeFalse())
 			Expect(hdr.DestConnectionID).To(Equal(connID))
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.KeyPhase).To(Equal(protocol.KeyPhaseZero))
 			Expect(extHdr.DestConnectionID).To(Equal(connID))
@@ -433,7 +473,7 @@ var _ = Describe("Header Parsing", func() {
 			hdr, _, _, err := ParsePacket(data, 5)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(hdr.IsLongHeader).To(BeFalse())
-			extHdr, err := hdr.ParseExtended(bytes.NewReader(data), versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(bytes.NewReader(data), protocol.Version1)
 			Expect(err).To(MatchError(ErrInvalidReservedBits))
 			Expect(extHdr).ToNot(BeNil())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0x42)))
@@ -449,7 +489,7 @@ var _ = Describe("Header Parsing", func() {
 			Expect(hdr.IsLongHeader).To(BeFalse())
 			Expect(hdr.DestConnectionID).To(Equal(connID))
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.KeyPhase).To(Equal(protocol.KeyPhaseZero))
 			Expect(extHdr.DestConnectionID).To(Equal(connID))
@@ -467,7 +507,7 @@ var _ = Describe("Header Parsing", func() {
 			Expect(err).ToNot(HaveOccurred())
 			Expect(hdr.IsLongHeader).To(BeFalse())
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.KeyPhase).To(Equal(protocol.KeyPhaseOne))
 			Expect(b.Len()).To(BeZero())
@@ -482,7 +522,7 @@ var _ = Describe("Header Parsing", func() {
 			hdr, _, _, err := ParsePacket(data, 4)
 			Expect(err).ToNot(HaveOccurred())
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.IsLongHeader).To(BeFalse())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0x1337)))
@@ -499,7 +539,7 @@ var _ = Describe("Header Parsing", func() {
 			hdr, _, _, err := ParsePacket(data, 10)
 			Expect(err).ToNot(HaveOccurred())
 			b := bytes.NewReader(data)
-			extHdr, err := hdr.ParseExtended(b, versionIETFFrames)
+			extHdr, err := hdr.ParseExtended(b, protocol.Version1)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(extHdr.IsLongHeader).To(BeFalse())
 			Expect(extHdr.PacketNumber).To(Equal(protocol.PacketNumber(0x99beef)))
@@ -530,12 +570,17 @@ var _ = Describe("Header Parsing", func() {
 				data = data[:i]
 				hdr, _, _, err := ParsePacket(data, 6)
 				Expect(err).ToNot(HaveOccurred())
-				_, err = hdr.ParseExtended(bytes.NewReader(data), versionIETFFrames)
+				_, err = hdr.ParseExtended(bytes.NewReader(data), protocol.Version1)
 				Expect(err).To(Equal(io.EOF))
 			}
 		})
 	})
 
+	It("distinguishes long and short header packets", func() {
+		Expect(IsLongHeaderPacket(0x40)).To(BeFalse())
+		Expect(IsLongHeaderPacket(0x80 ^ 0x40 ^ 0x12)).To(BeTrue())
+	})
+
 	It("tells its packet type for logging", func() {
 		Expect((&Header{IsLongHeader: true, Type: protocol.PacketTypeHandshake}).PacketType()).To(Equal("Handshake"))
 		Expect((&Header{}).PacketType()).To(Equal("1-RTT"))
