@@ -33,12 +33,10 @@ func checkEncoding(data []byte, expected map[string]interface{}) {
 	ExpectWithOffset(1, m).To(HaveLen(len(expected)))
 	for key, value := range expected {
 		switch v := value.(type) {
-		case string:
+		case bool, string, map[string]interface{}:
 			ExpectWithOffset(1, m).To(HaveKeyWithValue(key, v))
 		case int:
 			ExpectWithOffset(1, m).To(HaveKeyWithValue(key, float64(v)))
-		case bool:
-			ExpectWithOffset(1, m).To(HaveKeyWithValue(key, v))
 		case [][]float64: // used in the ACK frame
 			ExpectWithOffset(1, m).To(HaveKey(key))
 			for i, l := range v {
